/*
 * Filesystem image access functions.
 *
 * Copyright (C) 2019, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#ifndef __IMAGE_H__
#define __IMAGE_H__

#include <ext2fs/ext2fs.h>
#include <sys/stat.h>

#ifdef __cplusplus
extern "C" {
#endif

errcode_t image_create_file(ext2_filsys fs, ext2_ino_t ino_target,
                            const char *basename, __u16 mode,
                            __u16 uid, __u16 gid, ext2_ino_t *ino_file);

int image_file_type(int mode);

errcode_t image_find_next(ext2_filsys fs, ext2_ino_t ino_dir,
                          const char **basename, char *buf, ext2_ino_t *ino);

errcode_t image_find_path(ext2_filsys fs, const char **pathname,
                          ext2_ino_t *ino);

errcode_t image_find_file(ext2_filsys fs, const char *dirname,
                          const char *basename, ext2_ino_t *ino);

errcode_t image_inode(ext2_filsys fs, const char *pathname,
                      struct ext2_inode *inode);

errcode_t image_list_dir(ext2_filsys fs, const char *path,
                         int (*proc)(struct ext2_dir_entry *, int, int, char *,
                                     void *),
                         void *data);

errcode_t image_make_dir(ext2_filsys fs, ext2_ino_t ino_dir,
                         const char *basename, __u16 mode,
                         __u16 uid, __u16 gid, ext2_ino_t *ino);

errcode_t image_make_next_dir(ext2_filsys fs, ext2_ino_t ino_dir,
                              const char **basename, __u16 mode, __u16 uid,
                              __u16 gid, ext2_ino_t *ino);

errcode_t image_make_dirs(ext2_filsys fs, const char **pathname,
                          ext2_ino_t ino_dir, __u16 mode, __u16 uid,
                          __u16 gid);

errcode_t image_remove_by_inode(ext2_filsys fs, ext2_ino_t ino);

errcode_t image_rename(ext2_filsys fs, ext2_ino_t source,
                       ext2_ino_t source_parent, const char *source_basename,
                       ext2_ino_t target_parent, const char *target_basename);

void image_set_metadata(struct ext2_inode *inode, int clean, __u16 mode,
                        __u16 uid, __u16 gid);

errcode_t image_stat_inode(ext2_filsys fs, ext2_ino_t ino, struct stat *st);

errcode_t image_unlink_by_name(ext2_filsys fs, ext2_ino_t ino_parent,
                               const char *basename);

errcode_t image_unlink_by_inode(ext2_filsys fs, ext2_ino_t ino_parent,
                                ext2_ino_t ino);

int _image_isdir(ext2_filsys fs, ext2_ino_t ino);

int image_isdir(ext2_filsys fs, const char *name);

int _image_isfile(ext2_filsys fs, ext2_ino_t ino);

int image_isfile(ext2_filsys fs, const char *name);

#ifdef __cplusplus
}
#endif


#endif /* __IMAGE_H__ */
